package Converter::XmpDataToIptcData;

use version; our $VERSION = qv('0.4_5');
use strict;
use Util::Logfile;
use Text::Iconv;

use XMP::Reader;
use Util::Logfile;

require 'globals.pl';

sub new {
    my ( $class, $convert_to ) = @_;

    my $self = {};

    bless $self, $class;
    $self->{convert_to} = $convert_to;
    $self->_init();

    return $self;
}

sub _init {
    my ($self) = @_;

    $main::log = Util::Logfile->new() if not defined $main::log;
    undef $self->{iptc_data_hash_ref} if defined $self->{iptc_data_hash_ref};
    $self->{iptc_data_hash_ref}->{text_encoding} = $self->{convert_to};
}

sub get_iptc_data_from_xmp_data {
    my ( $self, $xmp_data_ref ) = @_;

    return if not defined $xmp_data_ref;

    $self->_init();
    $self->{xmp_data_ref} = $xmp_data_ref;

    if ( !$self->_is_convert() ) {
        $self->{iptc_data_hash_ref}->{text_encoding} = $self->{xmp_data_ref}->{text_encoding};
    }

    $self->_set_values();

    return $self->{iptc_data_hash_ref};
}

sub _set_values {
    my ($self) = @_;

    foreach my $xmp_key ( keys %{ $self->{xmp_data_ref} } ) {
        next if not defined $main::IPTC_KEY_FOR_XMP_KEY{$xmp_key};
        if ( main::has_xmp_key_multiple_iptc_keys($xmp_key) ) {
            foreach my $iptc_key ( @{ $main::IPTC_KEY_FOR_XMP_KEY{$xmp_key} } ) {
                $self->_set_xmp_to_iptc( $xmp_key, $iptc_key,
                    main::is_iptc_value_repeatable($iptc_key), undef );
            }
        }
        else {
            my $iptc_key = $main::IPTC_KEY_FOR_XMP_KEY{$xmp_key};
            $self->_set_xmp_to_iptc(
                $xmp_key, $iptc_key,
                main::is_iptc_value_repeatable($iptc_key),
                main::get_max_bytes_for_iptc_value($iptc_key)
            );
        }
    }

    $self->_set_splitted_values();
}

sub _set_xmp_to_iptc {
    my ( $self, $xmp_key, $iptc_key, $iptc_repeatable, $max_bytes ) = @_;
    my $xmp_repeatable = main::is_xmp_value_repeatable($xmp_key);
    my @xmp_array;
    my $xmp_value;

    if ($xmp_repeatable) {
        @xmp_array = @{ $self->{xmp_data_ref}->{$xmp_key} };
        $xmp_value = $xmp_array[0];
    }
    else {
        $xmp_value = $self->{xmp_data_ref}->{$xmp_key};
        push @xmp_array, $xmp_value;
    }

    if ($iptc_repeatable) {
        $self->_convert_array( \@xmp_array, $max_bytes );
        $main::log->log_verbose("Converter::XmpDataToIptcData: Setze XMP.$xmp_key -> IPTC.$iptc_key, Wert '@xmp_array'");
        $self->{iptc_data_hash_ref}->{$iptc_key} = \@xmp_array;
    }
    else {
        $main::log->log_verbose("Converter::XmpDataToIptcData: Setze XMP.$xmp_key -> IPTC.$iptc_key Wert '$xmp_value'");
        $self->{iptc_data_hash_ref}->{$iptc_key}
            = $self->_get_converted( $xmp_value, $max_bytes );
    }
}

sub _set_splitted_values {
    my ($self) = @_;

    $self->_set_iptc_creation_date();
    $self->_set_iptc_creation_time();
}

sub _set_iptc_creation_date {
    my ($self)                 = @_;
    my $iptc_creation_date_key = '2:62 Digital Creation Date';
    my $xmp_date_time          = $self->{iptc_data_hash_ref}->{$iptc_creation_date_key};

    return if not defined $xmp_date_time;

    $self->{iptc_data_hash_ref}->{$iptc_creation_date_key}
        = _get_iptc_creation_date($xmp_date_time);
}

sub _set_iptc_creation_time {
    my ( $self, $xmp_data_ref ) = @_;
    my $iptc_creation_time_key = '2:63 Digital Creation Time';
    my $xmp_date_time          = $self->{iptc_data_hash_ref}->{$iptc_creation_time_key};

    return if not defined $xmp_date_time;

    $self->{iptc_data_hash_ref}->{$iptc_creation_time_key}
        = _get_iptc_creation_time($xmp_date_time);
}

sub _get_iptc_creation_date {
    my ($xmp_date_time) = @_;

    # '2007-05-19T16:25:50.10+02:00' -> '2007-05-19'
    my $iptc_date = substr $xmp_date_time, 0, 10;

    $iptc_date =~ s/-//g;

    return $iptc_date;
}

sub _get_iptc_creation_time {
    my ($xmp_date_time) = @_;
    my $time            = _get_time($xmp_date_time);
    my $utc_diff        = _get_utc_diff($xmp_date_time);

    $time     =~ s/://g;
    $utc_diff =~ s/://g;

    return $time . $utc_diff;
}

sub _get_time {
    my ($xmp_date_time) = @_;

    # '2007-05-19T16:25:50.10+02:00' -> '16:25:50'
    return substr $xmp_date_time, 11, 8;
}

sub _get_utc_diff {
    my ($xmp_date_time) = @_;
    my $length_date = length $xmp_date_time;

    # '2007-05-19T16:25:50.10+02:00' -> '+02:00'
    my $start_index = $length_date - 6;

    return substr $xmp_date_time, $start_index, 6;
}

sub _get_converted {
    my ( $self, $string, $max_bytes ) = @_;

    return $string if !$self->_is_convert();

    my $converter = Text::Iconv->new( $self->{xmp_data_ref}->{text_encoding},
        $self->{convert_to} );

    return _get_max_bytes( $converter->convert($string), $max_bytes );
}

sub _convert_array {
    my ( $self, $array_ref, $max_bytes ) = @_;

    return if !$self->_is_convert();

    my $element_count = @{$array_ref};
    for ( my $index = 0; $index < $element_count; $index++ ) {
        my $element_ref = \$array_ref->[$index];
        ${$element_ref} = $self->_get_converted( ${$element_ref}, $max_bytes );
    }
}

sub _is_convert {
    my ($self) = @_;

    return $self->{convert_to}
        && $self->{xmp_data_ref}
        && $self->{xmp_data_ref}->{text_encoding}
        && $self->{xmp_data_ref}->{text_encoding} ne $self->{convert_to};
}

sub _get_max_bytes {
    my ( $string, $max_bytes ) = @_;

    return $string if not $max_bytes;
    return $string if length $string <= $max_bytes;

    $main::log->log_print("'$string' wird auf $max_bytes Zeichen verkuerzt!");
    return substr $string, 0, $max_bytes;
}

1;

__END__

=head1 NAME

Converter::XmpDataToIptcData - Konvertiert einen Hash mit XMP-Daten in einen
Hash mit IPTC-Daten

=head1 DESCRIPTION

Ein C<Converter::XmpDataToIptcData>-Objekt konvertiert einen Hash mit XMP-Daten
in einen Hash mit IPTC-Daten.

=head1 METHODS

=over

=item $converter = Converter::XmpDataToIptcData->new($convert_to);

Erzeugt ein neues Objekt.

Der 1. Parameter ist optional und ein String mit dem Zeichensatz, in den
alle XMP-Texte konvertiert werden sollen. Erlaubte Wert sind B<'UTF-8'>
und B<'ISO-8859-1'>.

=item $has_href = $converter->get_iptc_data_from_xmp_data($xmp_data)

Liefert einen Hash mit IPTC-Daten. 

Der 1. Parameter ist eine Referenz auf einen Hash mit den XMP-Daten

Die Rckgabe ist ein Hash mit IPTC-Daten oder undef.

Die Schlssel des Hashes sind aufgelistet im Dokument
L<http://www.elmar-baumann.de/fotografie/tipps/computer/lightroom/xmp-iptc-mapping.html>.

=back

=head1 AUTHOR

Elmar Baumann <eb@elmar-baumann.de> 2007/10/14

=cut
