package IPTC::ExifToolReader;

use version; our $VERSION = qv('0.3_5');
use strict;
use Readonly;
use Image::ExifTool qw(:Public);

use Util::Logfile;
use Util::Array;

require 'globals.pl';

Readonly my @raw_values => ( 'DateCreated', 'TimeCreated', );

sub new {
    my ($class) = @_;
    my $self = {};

    bless $self, $class;
    $main::log = Util::Logfile->new() if not defined $main::log;

    return $self;
}

sub get_iptc_data {
    my ($self) = @_;

    return $self->{iptc_data_hash_ref};
}

sub read_file {
    my ( $self, $filename ) = @_;

    $main::log->fail("Dateiname fehlt!")                   if not $filename;
    $main::log->fail("Datei '$filename' existiert nicht!") if not -f $filename;

    undef $self->{iptc_data_hash_ref} if defined $self->{iptc_data_hash_ref};
    $self->{exif_tool} = new Image::ExifTool();
    $self->{exif_tool}->Options( Charset => 'Latin', List => 1 );
    $main::log->log_verbose("IPTC::ExifToolReader: ImageInfo('$filename')");
    $self->{image_info} = $self->{exif_tool}->ImageInfo($filename);
    $self->_set_iptc_data();
}

sub _set_iptc_data {
    my ($self) = @_;

    foreach my $iptc_key ( keys %main::IPTC_INFO ) {
        my $exiftool_key = $main::IPTC_INFO{$iptc_key}->{ExifTool};
        next if not defined $exiftool_key;
        my $conf_method = _get_conf_method($exiftool_key);
        my $value = $self->{exif_tool}->GetValue( $exiftool_key, $conf_method );
        next if not defined $value;
        my $value_scalar;
        my $value_array_ref;
        if ( ref $value && ref $value eq 'ARRAY' ) {
            $value_array_ref = $value;
            $value_scalar = join " ", @{$value_array_ref};
        }
        elsif ( ref \$value eq 'SCALAR' ) {
            $value_array_ref = [$value];
            $value_scalar    = $value;
        }
        my $is_repeatable = main::is_iptc_value_repeatable($iptc_key);

        $self->{iptc_data_hash_ref}->{$iptc_key}
            = $is_repeatable ? $value_array_ref : $value_scalar;
    }
}

sub _get_conf_method {
    my ($exiftool_key) = @_;

    return Util::Array::exists( \@raw_values, $exiftool_key )
        ? 'Raw'
        : 'ValueConf';
}

1;

__END__

=head1 NAME

IPTC::ExifToolReader - Liest Informationen aus IPTC-Headern

=head1 DESCRIPTION

Die Klasse C<IPTC::ExifToolReader> liest Informationen aus IPTC-Headern von
Bilddateien. Sie benutzt dafr C<Image::ExifTool>
(L<http://owl.phy.queensu.ca/~phil/exiftool/>).

=head1 METHODS

=over

=item $iptc_reader = new IPTC::ExifToolReader()

Erzeugt ein neues Objekt

=item $iptc_reader->read_file($filename)

Liest den IPTC-Header einer Bilddatei.

Der 1. Parameter ist ein String mit dem Dateinamen

=item $iptc_data_hash_ref = $iptc_reader->get_iptc_data();

Liefert einen Hash mit den gelesenen IPTC-Daten oder undef, wenn keine IPTC-Daten
existieren oder gelesen wurden. Die Schlssel des Hashes stehen im Dokument
L<http://www.elmar-baumann.de/fotografie/tipps/computer/lightroom/xmp-iptc-mapping.html>.

=back

=head1 AUTHOR

Elmar Baumann <eb@elmar-baumann.de> 2007/05/19

=cut
