package IPTC::ExifToolWriter;

use version; our $VERSION = qv('0.4_0');
use strict;
use File::Copy;
use Image::ExifTool qw(:Public);

use Util::Logfile;
use Image::File;

require 'globals.pl';

sub new {
    my ($class) = @_;
    my $self = {};

    bless $self, $class;
    $main::log = Util::Logfile->new() if not defined $main::log;

    return $self;
}

sub can_write {
    my ( $self, $image_filename ) = @_;

    return Image::ExifTool::CanWrite($image_filename);
}

sub write_file {
    my ( $self, $image_filename, $iptc_data_hash_ref ) = @_;
    my $exif_tool = new Image::ExifTool();

    $main::log->fail("Benoetige IPTC-Daten!")
        if not defined $iptc_data_hash_ref;
    $main::log->fail("Dateiname fehlt!") if not $image_filename;
    $main::log->fail(
        "'Dateiformat von $image_filename' kann nicht geschrieben werden!")
        if not $self->can_write($image_filename);
    $main::log->fail("Datei '$image_filename' existiert nicht!") if not -f $image_filename;

    $self->{iptc_data_hash_ref} = $iptc_data_hash_ref;
    $exif_tool->Options( Charset => 'Latin' );

    return $self->_set_iptc_data($exif_tool)
        && $self->_write_file( $exif_tool, $image_filename );
}

sub _write_file {
    my ( $self, $exif_tool, $image_filename ) = @_;
    my $tmp_filename = "$image_filename-iptc-exiftoolwriter-tmp";

    $main::log->fail("Dateiname fehlt!")                   if not $image_filename;
    $main::log->fail("Datei '$image_filename' existiert nicht!") if not -f $image_filename;

    if ( $exif_tool->WriteInfo( $image_filename, $tmp_filename ) ) {
        move( $tmp_filename, $image_filename );
        $self->_write_xmp_into_imagefile( $exif_tool, $image_filename );
        $self->_modify_file_date($image_filename);
        return 1;
    }
    else {
        _print_exiftool_error($exif_tool);
        if ( -f $tmp_filename ) {
            unlink $tmp_filename;
        }
    }
    return 0;
}

sub _write_xmp_into_imagefile {
    my ( $self, $exif_tool, $image_filename ) = @_;

    if ($main::WRITE_XMP_INTO_IMAGEFILE) {
        my $tmp_filename = "$image_filename-iptc-exiftoolwriter-tmp";
        my $xmp_file     = Image::File::get_xmp_file_name($image_filename);
        if ( not -f $xmp_file ) {
            _print_exiftool_error(
                "IPTC::ExifToolWriter: XMP-Datei '$xmp_file' zu Bilddatei '$image_filename' existiert nicht!"
            );
            return;
        }
        $main::log->log_verbose(
            "IPTC::ExifToolWriter: Schreibe XMP-Informationen aus '$xmp_file' in Bilddatei '$image_filename'"
        );
        $exif_tool->Options( Charset => 'UTF8' );
        $exif_tool->SetNewValuesFromFile($xmp_file);
        if ( $exif_tool->WriteInfo( $image_filename, $tmp_filename ) ) {
            move( $tmp_filename, $image_filename );
        }
        else {
            _print_exiftool_error($exif_tool);
            if ( -f $tmp_filename ) {
                unlink $tmp_filename;
            }
        }
    }
}

sub _print_exiftool_error {
    my ($exif_tool) = @_;

    $main::log->log_print( $exif_tool->GetValue('Error') );
    $main::log->log_print( $exif_tool->GetValue('Warning') );
}

sub _set_iptc_data {
    my ( $self, $exif_tool ) = @_;

    foreach my $iptc_key ( keys %main::IPTC_INFO ) {
        my $exiftool_key = $main::IPTC_INFO{$iptc_key}->{ExifTool};
        next if not defined $exiftool_key;
        my $iptc_value = $self->{iptc_data_hash_ref}->{$iptc_key};
        if ( $iptc_key eq '2:62 Digital Creation Date' ) {
            $iptc_value = _convert_creation_date($iptc_value);
        }
        elsif ( $iptc_key eq '2:63 Digital Creation Time' ) {
            $iptc_value = _convert_creation_time($iptc_value);
        }
        $main::log->log_verbose(
            "IPTC::ExifToolWriter: SetNewValue($exiftool_key => '$iptc_value', Replace => 1)"
        );
        if (!$exif_tool->SetNewValue(
                $exiftool_key => $iptc_value,
                Replace       => 1
            )
            )
        {
            _print_exiftool_error($exif_tool);
            return 0;
        }
    }

    return 1;
}

sub _convert_creation_date {
    my ($creation_date) = @_;

    return undef if not defined $creation_date;
    return undef if ( length $creation_date != 8 );

    return
        substr( $creation_date,   0, 4 ) . ':'
        . substr( $creation_date, 4, 2 ) . ':'
        . substr( $creation_date, 6, 2 );
}

sub _convert_creation_time {
    my ($creation_time) = @_;

    return undef if not defined $creation_time;
    return undef if ( length $creation_time != 11 );

    return
        substr( $creation_time,   0, 2 ) . ':'
        . substr( $creation_time, 2, 2 ) . ':'
        . substr( $creation_time, 4, 5 ) . ':'
        . substr( $creation_time, 9, 2 );
}

sub _modify_file_date {
    my ( $self, $image_filename ) = @_;

    if ($main::SET_IMAGEFILE_MODIFY_TIME_FROM_SHOOTING_TIME) {
        my $exif_tool = Image::ExifTool->new();
        my $image_info = ImageInfo( $image_filename, 'DateTimeOriginal' );

        $exif_tool->SetNewValue(
            "FileModifyDate" => $image_info->{'DateTimeOriginal'},
            Protected        => 1
        );
        $main::log->log_verbose(
            "Setze Aufnahmezeit als Dateimodifikationsdatum bei '$image_filename");
        $exif_tool->SetFileModifyDate($image_filename);
    }
    Image::File::set_equal_time_to_xmp_file($image_filename);
}

1;

__END__

=head1 NAME

IPTC::ExifToolWriter - Schreibt IPTC-Daten in Dateien mit C<Image::ExifTool>

=head1 DESCRIPTION

Die Klasse C<IPTC::ExifToolWriter> schreibt IPTC-Daten in Dateien mit
C<Image::ExifTool> (L<http://owl.phy.queensu.ca/~phil/exiftool/>).

=head1 METHODS

=over

=item $iptc_writer = new IPTC::ExifToolWriter()

Erzeugt ein neues Objekt

=item $can_write = $iptc_writer->can_write($image_filename)

Liefert, ob IPTC-Daten in eine Datei geschrieben werden knnen; d.h.,
ob die C<Image::ExifTool> in eine Datei dieses Formats IPTC-Daten schreiben
knnen.

Das Dateiformat wird der Einfachheit halber anhand der Endung erkannt.

Der 1. Parameter ist ein String mit dem Namen der Bilddatei

Die Rckgabe ist true, wenn C<Image::ExifTool> IPTC-Daten in die Datei
schreiben kann

=item $iptc_writer->write_file($image_filename, $iptc_data_hash_ref)

Schreibt IPTC-Daten in eine Datei.

Der 1. Parameter ist ein String mit dem Dateinamen

Der 2. Parameter ist ein Hash mit den zu schreibenden IPTC-Daten

Die Rckgabe ist true bei erfolgreichem Schreiben

=back

=head1 AUTHOR

Elmar Baumann <eb@elmar-baumann.de> 2007/05/18

=cut
