package IPTC::ExifUtilsReader;

use version; our $VERSION = qv('0.3_0');
use strict;
use Readonly;

use Util::Logfile;
use Util::System;
use Util::Array;
use Util::String;

require 'globals.pl';

sub new {
    my ( $class, $option_praefix ) = @_;
    my $self = {};

    bless $self, $class;
    $self->{option_praefix} = $option_praefix;
    $main::log = Util::Logfile->new() if not defined $main::log;

    return $self;
}

sub get_iptc_data {
    my ($self) = @_;

    return $self->{iptc_data_hash_ref};
}

sub read_file {
    my ( $self, $filename ) = @_;

    $main::log->fail("Dateiname fehlt!")                   if not $filename;
    $main::log->fail("Datei '$filename' existiert nicht!") if not -f $filename;

    undef $self->{iptc_data_hash_ref} if defined $self->{iptc_data_hash_ref};
    $self->{filename} = $filename;
    $self->_set_iptc_data();
}

sub _set_iptc_data {
    my ($self) = @_;

    foreach my $iptc_key ( keys %main::IPTC_INFO ) {
        my $exifutils_option = $main::IPTC_INFO{$iptc_key}->{ExifUtils};
        my $value;
        $value = $self->_get_value( $exifutils_option,
            main::is_iptc_value_repeatable($iptc_key) );
        next if not defined $value;
        $self->{iptc_data_hash_ref}->{$iptc_key} = $value;
    }
    $self->_set_iptc_date();
}

sub _set_iptc_date {
    my ($self)            = @_;
    my $KEY_CREATION_DATE = '2:62 Digital Creation Date';
    my $date              = $self->{iptc_data_hash_ref}->{$KEY_CREATION_DATE};

    return if not defined $date;

    $date =~ s/\///g;

    $self->{iptc_data_hash_ref}->{$KEY_CREATION_DATE} = $date;

}

sub _get_value {
    my ( $self, $key, $as_array ) = @_;
    my $get_value_command = $self->_get_get_value_command($key);
    my $value             = Util::System::execute($get_value_command);

    $main::log->log_verbose(
        "IPTC::ExifUtilsReader: Hole Wert mit Befehl: '$get_value_command'");
    Util::String::trim( \$value );
    return undef if ( length $value eq 0 );

    if ($as_array) {
        my @array = split( /\;/, $value );
        Util::Array::trim_all_strings( \@array );
        return \@array;
    }
    return $value;
}

sub _get_get_value_command {
    my ( $self, $key ) = @_;

    return
        "exiflist  $self->{option_praefix}o l  $self->{option_praefix}f $key "
        . $self->{filename};
}

1;

__END__

=head1 NAME

IPTC::ExifUtilsReader - Liest Informationen aus IPTC-Headern

=head1 DESCRIPTION

Die Klasse C<IPTC::ExifUtilsReader> liest Informationen aus IPTC-Headern von
Bilddateien. Sie benutzt dafr die C<EXIFutils>
(L<http://www.hugsan.com/EXIFutils/>).

I<Das Programm C<exiflist> der Utils muss im Pfad fr ausfhrbare Dateien
stehen!>

=head1 METHODS

=over

=item $iptc_reader = new IPTC::ExifUtilsReader($option_praefix_string)

Erzeugt ein neues Objekt

Der 1. Parameter ist ein String mit dem Optionsprfix - einem Bindestrich
unter Linux und einem Schrgstrich unter Windows

=item $iptc_reader->read_file($filename)

Liest den IPTC-Header einer Bilddatei.

Der 1. Parameter ist ein String mit dem Dateinamen

=item $iptc_data_hash_ref = $iptc_reader->get_iptc_data();

Liefert einen Hash mit den IPTC-Daten.

Die Schlssel des Hashes sind aufgelistet im Dokument
L<http://www.elmar-baumann.de/fotografie/tipps/computer/lightroom/xmp-iptc-mapping.html>,
Spalte B<IPTC>.

=back

=head1 AUTHOR

Elmar Baumann <eb@elmar-baumann.de> 2007/10/13

=cut
