package XMP::Reader;

use version; our $VERSION = qv('0.4_6');
use strict;
use XML::Simple;
use Text::Iconv;
use Encode 'decode_utf8';

use Util::Array;
use Util::Object;
use Util::Logfile;

require 'globals.pl';

sub new {
    my ($class) = @_;
    my $self = {};

    bless $self, $class;
    $self->_reset();
    $main::log = Util::Logfile->new() if not defined $main::log;

    return $self;
}

sub _reset {
    my ($self) = @_;

    undef $self->{read_data_hash_ref} if defined $self->{read_data_hash_ref};
    $self->{read_data_hash_ref}->{text_encoding} = 'ISO-8859-1';
}

sub get_xmp_data {
    my ($self) = @_;

    return $self->{read_data_hash_ref};
}

sub read_file {
    my ( $self, $xmp_filename ) = @_;

    die "Dateiname fehlt!"                       if not defined $xmp_filename;
    die "Datei '$xmp_filename' existiert nicht!" if not -f $xmp_filename;

    $self->_reset();
    $self->{xmp_filename} = $xmp_filename;
    $main::log->log_verbose("Xmp::Reader: Lese Daten aus '$xmp_filename'");
    $self->_read_xmp_data();
    $self->_encode();
}

sub _read_xmp_data {
    my ($self) = @_;

    $self->{xml_in} = XMLin( $self->{xmp_filename}, ForceArray => 0 );
    if ( !defined $self->{xml_in}->{'rdf:RDF'} ) {
        $main::log->log_print(
            "Datei '$self->{xmp_filename}' enthlt nicht 'rdf:RDF'!", 1 );
        return;
    }

    foreach my $xmp_key ( keys %main::XMP_INFO ) {
        my $path_array_ref = $main::XMP_INFO{$xmp_key}->{Path};
        my $xmp_value      = $self->_get_value($path_array_ref);
        next if not defined $xmp_value;
        if (   main::is_xmp_value_repeatable($xmp_key)
            && Util::Object::is_type( 'SCALAR', $xmp_value ) )
        {
            $self->{read_data_hash_ref}->{$xmp_key} = [$xmp_value];
        }
        else {
            $self->{read_data_hash_ref}->{$xmp_key} = $xmp_value;
        }
    }
}

sub _get_value {
    my ( $self, $path_array_ref ) = @_;

    my $count_path_elements = @{$path_array_ref};
    my $count_iterations    = 0;
    my $ref                 = $self->{xml_in}->{'rdf:RDF'}->{'rdf:Description'};

    foreach my $path_element ( @{$path_array_ref} ) {
        return undef if not defined $ref;
        if ( ref $ref && ref $ref eq 'ARRAY' ) {
            $ref = Util::Array::get_hash_with_key_in_hash_array( $ref,
                $path_element );
            if ( $ref && ref $ref && ref $ref eq 'HASH' ) {
                $ref = $ref->{$path_element};
            }
        }
        elsif ( ref $ref && ref $ref eq 'HASH' ) {
            $ref = $ref->{$path_element};
        }
        else {
            return undef if $count_iterations < $count_path_elements;
        }
        $count_iterations++;
    }
    return $ref;
}

sub _encode {
    my ($self) = @_;

    foreach my $key ( keys %{ $self->{read_data_hash_ref} } ) {
        if (Util::Object::is_type(
                'SCALAR', $self->{read_data_hash_ref}->{$key}
            )
            )
        {
            _encode_scalar( \$self->{read_data_hash_ref}->{$key} );
        }
        elsif (
            Util::Object::is_type(
                'ARRAY', $self->{read_data_hash_ref}->{$key}
            )
            )
        {
            _encode_array( $self->{read_data_hash_ref}->{$key} );
        }
    }
}

# Kodiert UTF-8 nach ISO-8859-1
sub _encode_scalar {
    my ($scalar_ref) = @_;
    my $converter = Text::Iconv->new( "UTF-8", "ISO-8859-1" );

    $main::log->log_verbose(
        "Xmp::Reader: Kodiere '${$scalar_ref}' von UTF-8 nach ISO-8859-1");
    if ( decode_utf8( ${$scalar_ref} ) ) {
        ${$scalar_ref} = $converter->convert( ${$scalar_ref} );
    }
}

sub _encode_array {
    my ($array_ref) = @_;
    my $elment_count = @{$array_ref};

    for ( my $index = 0; $index < $elment_count; $index++ ) {
        _encode_scalar( \@{$array_ref}[$index] );
    }
}

1;

__END__

=head1 NAME

XMP::Reader - Liefert Daten aus XMP-Dateien

=head1 DESCRIPTION

Ein C<XMP::Reader>-Objekt parst XMP-Dateien und bietet Zugriff auf ganz
bestimmte Elemente darin. Es wird unterstellt, dass B<Adobe Lightroom>
die XMP-Dateien erzeugt hat. Der C<XMP::Reader> liefert Direktzugriff auf
alle ausgewhlten (gefilterten) Elemente; im Gegensatz zu C<XML::Simple>
muss kein Pfad traversiert werden.

Fr Elemente, die sich wiederholen knnen (B<main::is_xmp_value_repeatable>),
wird stets eine Array-Referenz geliefert, ansonsten ein String bzw. undef,
falls das Element nicht in der XMP-Datei ist.

Es werden lediglich Daten ausgelesen, die Lightroom schreibt und die auch in
die IPTC-Header von Bilddateien geschrieben werden knnen.

=head1 SYNOPSIS

 use XMP::Reader;
 
 my $xmp_reader = XMP::Reader->new();
 
 # XMP-Datei einlesen
 $xmp_reader->read_file('images/img_2555.xmp');
 $xmp_data = $xmp_reader->get_xmp_data();
 
 # Copyright holen
 $copyright = $xmp_data->{'dc:rights'}
 
 # Schlsselwrter holen
 @keywords = @{$xmp_data->{'dc:subject'}};

=head1 METHODS

=over

=item $xmp_reader = XMP::Reader->new()

Erzeugt ein neues Objekt.

=item $xmp_reader->read_file($xmp_filename)

Liest die Daten einer XMP-Datei.

Der 1. Parameter ist ein String mit dem Dateinamen 

=item $xmp_data = $xmp_reader->get_xmp_data()

Liefert eine Hash-Referenz mit den gelesenen XMP-Daten oder undef, wenn keine
Daten gefunden wurden. Die Schlssel im Hash referenzieren Strings, Referenzen
auf ein Array oder sind nicht definiert, falls nicht in der Datei vorhanden.

Alle mglichen Schlssel sind aufgelistet im Dokument
L<http://www.elmar-baumann.de/fotografie/tipps/computer/lightroom/mapping-xmp-iptc.html>,
Spalte B<XMP>.

=back

=head1 AUTHOR

Elmar Baumann <eb@elmar-baumann.de> 2007/05/17

=cut
